// Copyright (c) 2014, The Shelf REST project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

import 'package:shelf/shelf.dart';
import 'package:shelf/shelf_io.dart' as io;
import 'package:shelf_route/shelf_route.dart';
import 'package:shelf_rest/shelf_rest.dart';
import 'package:shelf_bind/shelf_bind.dart'; // TODO: reexport bind from rest

void main() {
//  var accountRouter = restRouter('/accounts', new AccountResource());
  var rootRouter = router()
    ..addAll(bindResource(new AccountResource()), path: '/accounts');

  var handler = const Pipeline()
      .addMiddleware(logRequests())
      .addHandler(rootRouter.handler);

  printRoutes(rootRouter);

  io.serve(handler, 'localhost', 8080).then((server) {
    print('Serving at http://${server.address.host}:${server.port}');
  });
}

class Account {
  final String accountId;
  final String name;

  Account.build({this.accountId, this.name});

  Account.fromJson(Map json) :
    this.accountId = json['accountId'],
    this.name = json['name'];

  Map toJson() => {
    'accountId' : accountId,
    'name' : name
  };
}

class AccountResourceModel extends ResourceModel<Account> {
  final Account account;
  
  AccountResourceModel(this.account, ResourceLinks links) : super(links);
  
  Map toJson() => super.toJson()..addAll({
    'account': account.toJson()
  });
  
}


@RestResource('accountId')
class AccountResource {
  
  // TODO: search support too
  List<Account> search(String name, [int page = 1]) {
    return [new Account.build(accountId: 'A999', name:name)];
  }

  AccountResourceModel create(@RequestBody() Account account, 
                              ResourceLinksFactory linksFactory) => 
      new AccountResourceModel(account, linksFactory(account.accountId));

  AccountResourceModel update(@RequestBody() Account account, 
                              ResourceLinksFactory linksFactory) => 
      new AccountResourceModel(account, linksFactory(account.accountId));

  @ResourceMethod(middleware: logRequests)
  AccountResourceModel find(String accountId, 
                            ResourceLinksFactory linksFactory) => 
      new AccountResourceModel(new Account.build(accountId: accountId),
          linksFactory(accountId));

  Map<dynamic, dynamic> childResources = {
    '/deposits' : new DepositResource()
  };
}

class Deposit {
  double amount;
  final String depositId;

  Deposit.build({this.depositId});

  Deposit.fromJson(Map json) : this.depositId = json['depositId'],
    this.amount = json['amount'];

  Map toJson() => {
      'depositId' : depositId,
      'amount' : amount
  };
}

@RestResource('depositId')
class DepositResource {

  @ResourceMethod(method: 'PUT')
  Deposit create(Deposit deposit) => deposit;
}



